/* cached_row_provider.cpp - abstract class wxPg::Cached_row_provider for
    accessing data from single cached row of SQL query result.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#include<wxPg/db/cached_row_provider.h>
#include<wxPg/utils/type.h>
#include<cstdlib>
#include<wxPg/utils/utils.h>


int
wxPg::Cached_row_provider::type_simple(const wxString &field){
    int col_idx = this->col_idx(field);
    if(col_idx < 0 || col_idx > this->cols())
        return TYPE_UNKNOWN;
    return this->type_simple(col_idx);
}


bool
wxPg::Cached_row_provider::null(const wxString &field){
    int col_idx = this->col_idx(field);
    if(col_idx < 0 || col_idx > this->cols())
        return true;
    return this->null(col_idx);
}


int
wxPg::Cached_row_provider::get_int(int idx){
    if(idx < 0 || idx > this->cols())
        return 0;
    switch(this->type_simple(idx)){
    case TYPE_BOOL:
        if(this->get_str(idx) == wxT("t"))
            return 1;
        return 0;
    default:
        return std::atoi(this->get_str(idx).char_str());
    }
}


int
wxPg::Cached_row_provider::get_int(const wxString &field){
    return this->get_int(this->col_idx(field));
}


double
wxPg::Cached_row_provider::get_float(int idx){
    if(idx < 0 || idx > this->cols())
        return 0.0;
    switch(this->type_simple(idx)){
    case TYPE_BOOL:
        if(this->get_str(idx) == wxT("t"))
            return 1.0;
        return 0.0;
    default:
        return std::atof(this->get_str(idx).char_str());
    }
}


double
wxPg::Cached_row_provider::get_float(const wxString &field){
    return this->get_float(this->col_idx(field));
}


const wxString
wxPg::Cached_row_provider::get_str(const wxString &field){
    int col_idx = this->col_idx(field);
    if(col_idx < 0 || col_idx > this->cols())
        return wxEmptyString;
    return this->get_str(col_idx);
}


const wxDateTime
wxPg::Cached_row_provider::get_date(int idx){
    wxDateTime result;
    if(idx < 0 || idx > this->cols())
        return result;
    switch(this->type_simple(idx)){
    case TYPE_DATE:
    case TYPE_TIMESTAMP:
    case TYPE_STRING:
        result.ParseDate(this->get_str(idx));
        break;
    }

    return result;
}


const wxDateTime
wxPg::Cached_row_provider::get_date(const wxString &field){
    return this->get_date(this->col_idx(field));
}


const wxDateTime
wxPg::Cached_row_provider::get_time(int idx){
    wxDateTime result;
    if(idx < 0 || idx > this->cols())
        return result;
    switch(this->type_simple(idx)){
    case TYPE_TIME:
        result.ParseTime(this->get_str(idx));
        break;
    case TYPE_STRING:
        {
            wxString value = this->get_str(idx);
            if(result.ParseDateTime(value))
                break;
            result.ParseTime(value);
        }
        break;
    case TYPE_TIMESTAMP:
        result.ParseDateTime(this->get_str(idx));
        break;
    }
    return result;
}


const wxDateTime
wxPg::Cached_row_provider::get_time(const wxString &field){
    return this->get_time(this->col_idx(field));
}


const wxDateTime
wxPg::Cached_row_provider::get_date_time(int idx){
    wxDateTime result;
    if(idx < 0 || idx > this->cols())
        return result;
    wxString value = this->get_str(idx);
    switch(this->type_simple(idx)){
    case TYPE_TIMESTAMP:
        result.ParseDateTime(value);
        break;
    case TYPE_DATE:
        result.ParseDate(value);
        break;
    case TYPE_TIME:
        result.ParseTime(value);
        break;
    case TYPE_STRING:
        if(result.ParseDateTime(value))
            break;
        if(result.ParseDate(value))
            break;
        result.ParseTime(value);
        break;
    }

    return result;
}


const wxDateTime
wxPg::Cached_row_provider::get_date_time(const wxString &field){
    return this->get_date_time(this->col_idx(field));
}


bool
wxPg::Cached_row_provider::get_bool(int idx){
    if(idx < 0 || idx > this->cols())
        return false;
    return wxPg::cstr_to_bool(this->get_str(idx).char_str(),
                              this->type_simple(idx));
}


bool
wxPg::Cached_row_provider::get_bool(const wxString &field){
    return this->get_bool(this->col_idx(field));
}


const wxString
wxPg::Cached_row_provider::format_cached_value(const char *value,
                                               int type,
                                               const Database &db)const{
    wxString val = db.char_to_str(value);
    switch(type){
    case TYPE_DATE:
        {
            wxDateTime date;
            date.ParseDate(val);
            val = date.FormatISODate();
            break;
        }
    case TYPE_TIME:
        {
            wxDateTime time;
            time.ParseTime(val);
            val = time.FormatISOTime();
            break;
        }
    case TYPE_TIMESTAMP:
        {
            wxDateTime timestamp;
            timestamp.ParseDateTime(val);
            val = timestamp.Format(wxT("%Y-%m-%d %H:%M:%S"));
            break;
        }
    }

    return val;
}
