/* query_browser.cpp - implementation of class wxPg::Query_browser.

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#include<wxPg/db/query_browser.h>
#include<libpq-fe.h>


wxPg::Query_browser::Query_browser(Database *db, unsigned buf_size)
    : Data_row(db), wxPg::Query(db), buffer_size_(buf_size), abs_pos_(0),
      total_recs_(0), abs_cursor_pos_(0), end_reached_(false)
{}


/* When abs_pos_ == current(), currently fetched buffer is the beginning
    of result. Othrwise we need to move to beginning and fetch buffer of
    rows.
    At the end set position variables to 0, to activate the first row.
*/
void
wxPg::Query_browser::first(){
    if(static_cast<long long>(this->abs_pos_)
            != static_cast<long long>(this->current())){
        if(!this->Query::move(0, FM_ABSOLUTE))
            return;
        if(!this->Query::fetch(this->buffer_size()))
            return;
        int rows = this->rows();
        if(rows < this->buffer_size()){
            this->end_reached_ = true;
            this->abs_cursor_pos_ = rows + 1;
        }else this->abs_cursor_pos_ = rows;
    }
    this->abs_pos_ = 0;
    this->current_ = 0;
}


/*
IF end_reached_ THEN
    check if not last record
    IF last buffer THEN
        make last record active
    ELSE
        fetch last buffer_size_ of rows
        set active row to last in buffer
    ENDIF
ELSE
    move to the end and get number of rows in result.
    move backwards by buffer_size_ + 1
    fetch last buffer_size_ of rows
    set active row to last in buffer
ENDIF
*/
void
wxPg::Query_browser::last(){
    if(this->end_reached_){
        if(this->abs_pos_ == this->total_recs_ - 1)
            return;
        if(this->abs_cursor_pos_ >= this->total_recs_){
            this->current_ = this->rows() - 1;
            this->abs_pos_ = this->total_recs_ - 1;
        }else{
            if(!this->Query::move(this->total_recs_ - this->buffer_size(),
                                  FM_ABSOLUTE))
                return;
            if(!this->Query::fetch(this->buffer_size()))
                return;
            this->abs_pos_ = this->total_recs_ - 1;
            this->abs_cursor_pos_ = this->total_recs_;
            this->current_ = this->rows() - 1;
        }
    }else{
        if(!this->Query::move(0, FM_FORWARD_ALL))
            return;
        this->abs_cursor_pos_ += strtoull(PQcmdTuples(this->result_),
                                                      NULL, 10) + 1;
        this->total_recs_ = this->abs_cursor_pos_ - 1;
        this->abs_pos_ = this->total_recs_ - 1;
        this->end_reached_ = true;
        if(!this->Query::move(this->buffer_size() + 1, FM_BACKWARD))
            return;
        if(!this->Query::fetch(this->buffer_size()))
            return;
        this->current_ = this->rows() - 1;
    }
}


/*
Check for end and return if true.
If in the midle of buffer, increment position and return.
Fetch next buffer
    go to last on error or no rows.
    update position state variables.
*/
bool
wxPg::Query_browser::next(){
    if(this->end_reached_ && this->abs_pos_ == this->total_recs_ - 1)
        return false;
    if(this->current() < this->rows() - 1){ //inside current buffer
        ++this->current_;
        ++this->abs_pos_;
    }else{
        if(!this->Query::fetch(this->buffer_size())){
            this->last();
            return false;
        }else{
            int rows = this->rows();
            if(!rows){
                this->last();
                return false;
            }
            this->abs_cursor_pos_ += rows;
            ++this->abs_pos_;
            if(this->abs_cursor_pos_ > this->total_recs_)
                this->total_recs_ = this->abs_cursor_pos_;
            if(unsigned(rows) < this->buffer_size()){
                ++this->abs_cursor_pos_;
                this->end_reached_ = true;
            }
        }
    }

    return true;
}


/*
Check for beginning of result.
If in the midle of buffer, decrement position variables.
Move backwards by all current buffer + buffer_size_.
Fetch buffer.
Update position variables.
*/
bool
wxPg::Query_browser::prev(){
    if(!this->abs_pos_)
        return false;
    if(this->current_){ //inside current buffer
        --this->current_;
        --this->abs_pos_;
    }else{
        int fetch_from = int(this->abs_cursor_pos_ - this->rows()
                             - this->buffer_size());
        if(fetch_from < 0)
            fetch_from = 0;
        if(!this->Query::move(fetch_from, FM_ABSOLUTE)){
            this->first();
            return false;
        }
        if(!this->Query::fetch(this->buffer_size())){
            return false;
        }

        this->abs_cursor_pos_ = fetch_from + this->rows();
        --this->abs_pos_;
        this->current_ = this->rows()
                         - (this->abs_cursor_pos_ - this->abs_pos_);
    }

    return true;
}



/*
Call parents open().
Fetch _buffer_size of rows.
Update position variables.
*/
bool
wxPg::Query_browser::open(const wxString &sql){
    //simple open from parent
    if(!this->Query::open(sql))
        return false;
    //need simple fetch operation, implemented in parent
    if(!this->Query::fetch(this->buffer_size()))
        return false;
    this->abs_pos_ = 0;
    int rows = this->rows();
    if(rows < this->buffer_size()){
        this->abs_cursor_pos_ = rows + 1;
        this->end_reached_ = true;
    }else{
        this->abs_cursor_pos_ = rows;
        this->end_reached_ = false;
    }
    this->total_recs_ = rows;
    return true;
}



bool
wxPg::Query_browser::fetch(int rows, int flag){
    switch(flag){
    case Query::FM_FIRST:
        return this->fetch_first_();
    case Query::FM_LAST:
        return this->fetch_last_();
    case Query::FM_ABSOLUTE:
        return this->fetch_absolute_(rows);
    case Query::FM_RELATIVE:
        return this->fetch_relative_(rows);
    case Query::FM_FORWARD:
        return this->fetch_forward_(rows);
    case Query::FM_FORWARD_ALL:
        return this->fetch_forward_all_();
    case Query::FM_BACKWARD:
        return this->fetch_backward_(rows);
    case Query::FM_BACKWARD_ALL:
        return this->fetch_backward_all_();
    }

    return false;
}


bool
wxPg::Query_browser::move(int rows, int flag){
    switch(flag){
    case FM_FORWARD_ALL:
        return this->move_forward_all_();
    case FM_FIRST:
        rows = 1;
        //continue
    case FM_ABSOLUTE:
        return this->move_absolute_(rows);
    case FM_RELATIVE:
        return this->move_relative_(rows);
    case FM_LAST:
        return this->move_last_();
    case FM_FORWARD:
        return this->move_forward_(rows);
    case FM_BACKWARD:
        return this->move_backward_(rows);
    case FM_BACKWARD_ALL:
        return this->move_backward_all_();
    default:
        return false;
    }
}


/*
IF parents current() succeeds THEN
    update _abs_pos
ENDIF
*/
bool
wxPg::Query_browser::current(int new_curr){
    int diff = new_curr - this->current_;
    if(this->Query::current(new_curr)){
        this->abs_pos_ += diff;
        return true;
    }

    return false;
}


bool
wxPg::Query_browser::go_to_record(unsigned long long row, unsigned buf_pos){
    if(row == this->abs_pos())  //already good position
        return true;
    unsigned long long abs_pos = this->abs_pos();
    if(this->go_to_record_(row, buf_pos))
        return true;
    this->fetch_from_(abs_pos);
    return false;
}


bool
wxPg::Query_browser::fetch_first_(){
    if(this->abs_pos_ == 0 && this->rows() == 1)
        return true;
    if(this->Query::fetch(0, FM_FIRST)){
        this->abs_cursor_pos_ = 1;
        this->abs_pos_ = 0;
        this->current_ = 0;
        return this->rows();
    }

    return false;
}


/*
If already last, then return.
IF end_reached THEN
    fetch lst
    update position variables
ELSE
    move forward all
    get record count
    fetch last
    update position variables
ENDIF
*/
bool
wxPg::Query_browser::fetch_last_(){
    if(this->end_reached_ && this->abs_pos_ == this->total_recs_ - 1
            && this->rows() == 1)
        return true;
    if(this->end_reached_){
        if(this->Query::fetch(0, FM_LAST)){
            if(this->rows() != 1)
                return false;
            this->abs_cursor_pos_ = this->total_recs_ + 1;
            this->abs_pos_ = this->total_recs_ - 1;
            this->current_ = 0;
            return true;
        }
    }else if(this->Query::move(0, FM_FORWARD_ALL)){
        this->abs_cursor_pos_ += strtoull(PQcmdTuples(this->result_),
                                          NULL, 10);
        if(!this->Query::fetch(0, FM_LAST))
            return false;
        if(this->rows() != 1)
            return false;
        this->abs_pos_ = this->abs_cursor_pos_ - 1;
        this->total_recs_ = this->abs_cursor_pos_;
        this->end_reached_ = true;
        return true;
    }

    return false;
}


/*
Check if not requesting record out of range.
IF requesting in known limits THEN
    fetch record
    update position variables
ELSE
    try to fetch record
    IF failed THEN
        move to original position
    ENDIF
    update position variables
ENDIF
*/
bool
wxPg::Query_browser::fetch_absolute_(int row){
    if(row <= 0 || (this->end_reached_ && row > this->total_recs_))
        return false;
    if(row <= this->total_recs_){
        if(!this->Query::fetch(row, FM_ABSOLUTE))
            return false;
        this->abs_cursor_pos_ = row;
        this->abs_pos_ = this->abs_cursor_pos_ - 1;
        this->current_ = 0;
        return true;
    }else{
        unsigned long long cursor_pos = this->abs_cursor_pos_;
        bool fetch_ok = this->Query::fetch(row, FM_ABSOLUTE);
        if(fetch_ok)
            fetch_ok = this->rows();
        if(fetch_ok)
            this->abs_cursor_pos_ = row;
        else if(!this->Query::fetch(cursor_pos, FM_ABSOLUTE))
            return false;
        else this->abs_cursor_pos_ = cursor_pos;
        this->abs_pos_ = this->abs_cursor_pos_ - 1;
        this->current_ = 0;
        if(this->abs_cursor_pos_ > this->total_recs_)
            this->total_recs_ = this->abs_cursor_pos_;
        return fetch_ok;
    }
}


bool
wxPg::Query_browser::fetch_relative_(int offset){
    return this->fetch_absolute_(this->abs_cursor_pos_ + offset);
}


/*
Fetch rows.
Update position.
*/
bool
wxPg::Query_browser::fetch_forward_(int rows){
    if(this->end_reached_ && this->abs_cursor_pos_ > this->total_recs_)
        return true;
    if(!this->Query::fetch(rows, FM_FORWARD))
        return false;
    int fetched = this->rows();
    if(fetched > 0){
        this->abs_cursor_pos_ += fetched;
        this->abs_pos_ = this->abs_cursor_pos_ - fetched;
        if(this->abs_cursor_pos_ >= this->total_recs_)
            this->total_recs_ = this->abs_cursor_pos_;
    }
    if(fetched < rows){
        this->end_reached_ = true;
        ++this->abs_cursor_pos_;
    }

    return true;
}


/*
Fetch all foreward.
Update position.
*/
bool
wxPg::Query_browser::fetch_forward_all_(){
    if(!this->Query::fetch(0, FM_FORWARD_ALL))
        return false;

    unsigned long long fetched = strtoull(PQcmdTuples(this->result_), NULL, 0);
    if(fetched){
        this->abs_pos_ = this->abs_cursor_pos_;
        this->abs_cursor_pos_ += fetched + 1;
        this->total_recs_ = this->abs_cursor_pos_ - 1;
    }else this->abs_cursor_pos_ = this->total_recs_ + 1;

    this->end_reached_ = true;
    return true;
}


/*
Move backward by rows + 1.
Fetch rows.
Update position variables.
*/
bool
wxPg::Query_browser::fetch_backward_(int rows){
    if(this->abs_cursor_pos_ == 0)
        return true;

    if(!this->Query::move(rows + 1, FM_BACKWARD))
        return false;
    int moved = strtol(PQcmdTuples(this->result_), NULL, 0);
    if(moved <= rows)
        rows = moved;
    if(rows <= 0){
        this->abs_cursor_pos_ = 0;
        return true;
    }
    if(!this->Query::fetch(rows))
        return false;

    --this->abs_cursor_pos_;
    this->abs_pos_ = this->abs_cursor_pos_ - 1;
    this->current_ = this->rows() - 1;

    return true;
}


/*
Move to position 0.
Fetch all befere previous current record.
*/
bool
wxPg::Query_browser::fetch_backward_all_(){
    if(this->abs_cursor_pos_ <= 1)
        return false;
    if(!this->Query::move(0, FM_BACKWARD_ALL))
        return false;
    if(!this->Query::fetch(int(this->abs_cursor_pos_ - 1)))
        return false;

    --this->abs_cursor_pos_;
    this->abs_pos_ = this->abs_cursor_pos_ - 1;
    this->current_ = this->rows() - 1;
    return true;
}


bool
wxPg::Query_browser::move_forward_all_(){
    if(this->end_reached_ && this->abs_cursor_pos_ > this->total_recs_)
        return true;
    if(!this->Query::move(0, FM_FORWARD_ALL))
        return false;
    this->abs_cursor_pos_ += strtoull(PQcmdTuples(this->result_),
                                      NULL, 10) + 1;
    this->total_recs_ = this->abs_cursor_pos_ - 1;
    this->end_reached_ = true;
    return true;
}


/*
move to record.
if failed, move to original record.
update position variables.
*/
bool
wxPg::Query_browser::move_absolute_(int rows){
    unsigned long long cursor_pos = this->abs_cursor_pos_;
    bool move_ok = this->Query::move(rows, FM_ABSOLUTE);
    if(move_ok)
        move_ok = strtoull(PQcmdTuples(this->result_), NULL, 10);
    if(!move_ok)
        this->Query::move(cursor_pos, FM_ABSOLUTE);
    else{
        this->abs_cursor_pos_ = rows;
        this->abs_pos_ = this->abs_cursor_pos_ - 1;
        if(this->abs_cursor_pos_ > this->total_recs_)
            this->total_recs_ = this->abs_cursor_pos_;
    }
    return move_ok;
}


/*
check if moving at all.
check if not moveing out of range.
move to record.
if failed, move to original.
update position variables.
*/
bool
wxPg::Query_browser::move_relative_(int rows){
    if(rows == 0)
        return true;
    if(rows < 0 && (this->abs_cursor_pos_ + rows) <= 0)
        return false;
    if(this->end_reached_
            && (this->abs_cursor_pos_ + rows) > this->total_recs_)
        return false;
    unsigned long long cursor_pos = this->abs_cursor_pos_;
    bool move_ok = this->Query::move(rows, FM_RELATIVE);
    if(move_ok)
        move_ok = strtoull(PQcmdTuples(this->result_), NULL, 10);
    if(!move_ok)
        this->Query::move(cursor_pos, FM_ABSOLUTE);
    else{
        this->abs_cursor_pos_ += rows;
        this->abs_pos_ = this->abs_cursor_pos_ - 1;
        if(this->abs_cursor_pos_ > this->total_recs_)
            this->total_recs_ = this->abs_cursor_pos_;
    }
    return move_ok;
}


/*
if not reached end yet, move forward by all records.
get total record count.
if previous move successfull, move to last.
if any previous move failed, move to original record.
update position variables.
*/
bool
wxPg::Query_browser::move_last_(){
    unsigned long long cursor_pos = this->abs_cursor_pos_;
    bool move_ok = true;
    if(!this->end_reached_){
        move_ok = this->Query::move(0, FM_FORWARD_ALL);
        if(move_ok){
            this->abs_cursor_pos_ += strtoull(PQcmdTuples(this->result_),
                                              NULL, 10);
            this->total_recs_ = this->abs_cursor_pos_;
            this->end_reached_ = true;
        }
    }
    if(move_ok)
        move_ok = this->Query::move(0, FM_LAST);
    if(!move_ok)
        this->move(cursor_pos, FM_ABSOLUTE);
    else{
        this->abs_cursor_pos_ = this->total_recs_;
        this->abs_pos_ = this->abs_cursor_pos_ - 1;
    }

    return move_ok;
}


/*
check if not at the end.
move forward.
if successfull, update position variables.
if failed to move, move to original record.
*/
bool
wxPg::Query_browser::move_forward_(int rows){
    if(this->end_reached_ && this->abs_cursor_pos_ + rows >= this->total_recs_)
        return false;
    unsigned long long cursor_pos = this->abs_cursor_pos_;
    bool move_ok = this->Query::move(rows, FM_FORWARD);
    if(move_ok){
        unsigned long long moved = strtoull(PQcmdTuples(this->result_),
                                            NULL, 10);
        if(moved < rows)
            move_ok = false;
        else{
            this->abs_cursor_pos_ += rows;
            this->abs_pos_ = this->abs_cursor_pos_ - 1;
            if(this->abs_cursor_pos_ > this->total_recs_)
                this->total_recs_ = this->abs_cursor_pos_;
        }
    }
    if(!move_ok)
        this->move(cursor_pos, FM_ABSOLUTE);
    return move_ok;
}


bool
wxPg::Query_browser::move_backward_(int rows){
    if(this->abs_cursor_pos_ == 0 || rows > this->abs_cursor_pos_)
        return false;
    if(this->Query::move(rows, FM_BACKWARD)){
        this->abs_cursor_pos_ -= rows;
        this->abs_pos_ = this->abs_cursor_pos_ ? this->abs_cursor_pos_ - 1 : 0;
        return true;
    }
    return false;
}


bool
wxPg::Query_browser::move_backward_all_(){
    if(this->abs_cursor_pos_ == 0)
        return true;
    if(this->Query::move(0, FM_BACKWARD_ALL)){
        this->abs_cursor_pos_ = 0;
        this->abs_pos_ = 0;
        return true;
    }
    return false;
}


/*
if inside current buffer, set requested record current.
else - fetch from requested record.
*/
bool
wxPg::Query_browser::go_to_record_(unsigned long long row, unsigned buf_pos){
    if(row < this->abs_pos()){
        unsigned long long buf_start = this->abs_pos() - this->current();
        if(buf_start <= row){ //inside current buffer
            this->current(row - buf_start);
            return true;
        }
    }else{
        unsigned buf_pos = row - (this->abs_pos() - this->current());
        if(buf_pos < this->rows()){
            this->current(buf_pos); //inside current buffer
            return true;
        }
    }

    if(buf_pos >= this->buffer_size())
        buf_pos %= this->buffer_size();
    unsigned long long fetch_from;
    if(buf_pos >= row){
        fetch_from = 0;
        buf_pos = row;
    }else fetch_from = row - buf_pos;

    if(!this->fetch_from_(fetch_from))
        return false;
    if(this->rows() < buf_pos)
        return false;
    if(this->abs_pos() != fetch_from)
        buf_pos += row - this->abs_pos();
    this->current(buf_pos);

    return true;
}


bool
wxPg::Query_browser::fetch_from_(unsigned long long row){
    int flag = row ? FM_ABSOLUTE : FM_BACKWARD_ALL;
    if(!this->move(row, flag))
        return false;
    if(!this->fetch(this->buffer_size()))
        return false;
    return this->rows();
}
