/* updatable_row_provider.h - class wxPg::Updatable_row for
    changing a single row of a database table.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#include<wxPg/db/updatable_row.h>
#include<wxPg/utils/type.h>
#include<wxPg/utils/utils.h>
#include<wx/intl.h>
#include<libpq-fe.h>


wxPg::Updatable_row::Updatable_row(Database *db)
: db_(db), loaded_(false)
{}


int
wxPg::Updatable_row::cols(){
    return this->data_.size();
}


const wxString
wxPg::Updatable_row::col_name(int idx){
    if(idx < 0 || idx > this->data_.size())
        return wxEmptyString;
    return this->data_[idx].name;
}


int
wxPg::Updatable_row::col_idx(const wxString &field){
    std::map<wxString, int>::iterator i = this->field_indices_.find(field);
    if(i == this->field_indices_.end())
        return -1;
    return i->second;
}


int
wxPg::Updatable_row::type_simple(int idx){
    if(idx < 0 || idx > this->data_.size())
        return TYPE_UNKNOWN;
    return this->data_[idx].type;
}


bool
wxPg::Updatable_row::null(int idx){
    if(idx < 0 || idx > this->data_.size())
        return true;
    return this->data_[idx].null;
}


const wxString
wxPg::Updatable_row::get_str(int idx){
    if(idx < 0 || idx > this->data_.size())
        return wxEmptyString;
    return this->data_[idx].value;
}


void
wxPg::Updatable_row::set_value(const wxString &field, int value){
    wxString val;
    val << value;
    this->set_field_(field, val, false, wxPg::TYPE_INT);
}


void
wxPg::Updatable_row::set_value(const wxString &field, double value){
    wxString val;
    val << value;
    this->set_field_(field, val, false, wxPg::TYPE_FLOAT);
}


void
wxPg::Updatable_row::set_value(const wxString &field, const wxString &value){
    this->set_field_(field, value, false, wxPg::TYPE_STRING);
}


void
wxPg::Updatable_row::set_value(const wxString &field, const wxDateTime &value){
    wxString val = value.Format(wxT("%Y-%m-%d %H:%M:%S"));
    this->set_field_(field, val, false, TYPE_TIMESTAMP);
}


void
wxPg::Updatable_row::set_date(const wxString &field, const wxDateTime &value){
    wxString val = value.FormatISODate();
    this->set_field_(field, val, false, TYPE_DATE);
}


void
wxPg::Updatable_row::set_time(const wxString &field, const wxDateTime &value){
    wxString val = value.FormatISOTime();
    this->set_field_(field, val, false, TYPE_TIME);
}


void
wxPg::Updatable_row::set_bool(const wxString &field, bool value){
    wxString val;
    if(value)
        val = wxT("t");
    else val = wxT("f");
    this->set_field_(field, val, false, TYPE_BOOL);
}


void
wxPg::Updatable_row::set_null(const wxString &field){
    this->set_field_(field, wxEmptyString, true, TYPE_UNKNOWN);
}


void
wxPg::Updatable_row::set_type_simple(const wxString &field, int type){
    if(this->loaded())
        return;
    if(this->null(field))
        this->set_field_(field, wxEmptyString, true, type);
    else switch(type){
    case TYPE_BOOL:
        {
            wxString value;
            if(this->get_bool(field))
                value = wxT("t");
            else value = wxT("f");
            this->set_field_(field, value, false, type);
        }
        break;
    case TYPE_INT:
        {
            wxString str_val = this->get_str(field);
            wxString value;
            value << this->get_int(field);
            if(str_val.Trim(false).StartsWith(value)
                    || this->type_simple(field) == TYPE_BOOL)
                this->set_field_(field, value, false, type);
            else this->set_field_(field, wxEmptyString, true, type);
        }
        break;
    case TYPE_FLOAT:
        {
            wxString str_val = this->get_str(field);
            wxString value;
            value << this->get_float(field);
            if(str_val.Trim(false).StartsWith(value)
                    || this->type_simple(field) == TYPE_BOOL)
                this->set_field_(field, value, false, type);
            else this->set_field_(field, wxEmptyString, true, type);
        }
        break;
    case TYPE_STRING:
        this->set_field_(field, this->get_str(field), false, type);
        break;
    case TYPE_DATE:
        {
            wxString str_val = this->get_str(field);
            wxDateTime date_val = this->get_date(field);
            wxString value = date_val.FormatISODate();
            if(str_val.StartsWith(value))
                this->set_field_(field, value, false, type);
            else this->set_field_(field, wxEmptyString, true, type);
        }
        break;
    case TYPE_TIME:
        {
            wxString str_val = this->get_str(field);
            wxDateTime time_val = this->get_time(field);
            wxString value = time_val.FormatISOTime();
            if(str_val.EndsWith(value))
                this->set_field_(field, value, false, type);
            else this->set_field_(field, wxEmptyString, true, type);
        }
        break;
    case TYPE_TIMESTAMP:
        {
            wxString str_val = this->get_str(field);
            wxDateTime date_time_val = this->get_date_time(field);
            wxString value = date_time_val.FormatISODate() + wxT(" ")
                             + date_time_val.FormatISOTime();
            if(value == str_val)
                this->set_field_(field, value, false, type);
            else if(value.StartsWith(str_val))  //when value is date only
                this->set_field_(field, value, false, type);
            else if(value.EndsWith(str_val))    //when value is time only
                this->set_field_(field, value, false, type);
            else this->set_field_(field, wxEmptyString, true, type);
        }
        break;
    }
}


void
wxPg::Updatable_row::clear(){
    if(this->loaded())
        for(unsigned i = 0; i < this->data_.size(); ++i){
            this->data_[i].value = wxEmptyString;
            this->data_[i].null = true;
        }
    else{
        this->data_.clear();
        this->field_indices_.clear();
    }
}


bool
wxPg::Updatable_row::load(const wxString &condition){
    this->unload();
    if(!this->database()){
        this->last_err_ = _("No database connection.");
        return false;
    }
    wxString sql = this->table();
    if(sql.Trim() == wxEmptyString){
        this->last_err_ = _("No table.");
        return false;
    }

    sql = wxT("select * from ") + sql + wxT(" where ");
    if(condition == wxEmptyString)
        sql += wxT("1=0");
    else sql += condition;

    PGresult *res = this->database()->run_(sql);
    if(!res){
        this->last_err_ = this->database()->last_err();
        return false;
    }

    bool result = this->load_(res);
    if(result){
        this->condition_ = condition;
        this->loaded_ = true;
        this->last_err_ = wxEmptyString;
    }
    PQclear(res);

    return result;
}


void
wxPg::Updatable_row::unload(){
    this->loaded_ = false;
    this->condition_ = wxEmptyString;
    this->clear(); //removes columns, since no longer in loaded mode
}


bool
wxPg::Updatable_row::insert(){
    if(!this->modify_check_())
        return false;

    wxString table = wxString(this->table()).Trim();
    wxString sql = wxT("insert into ") + table
                   + wxT("(") + this->data_[0].name;
    wxString sql2 = wxT(") values(") + this->sql_value_(0);
    for(unsigned i = 1; i < this->data_.size(); ++i){
        sql += wxT(", ") + this->data_[i].name;
        sql2 += wxT(", ") + this->sql_value_(i);
    }
    sql += sql2 + wxT(")");
    return this->run_(sql);
}


bool
wxPg::Updatable_row::update(const wxString &condition){
    if(!this->modify_check_())
        return false;

    wxString table = wxString(this->table()).Trim();
    wxString sql = wxT("update ") + table + wxT(" set ")
                   + this->data_[0].name + wxT(" = ") + this->sql_value_(0);
    for(unsigned i = 1; i < this->data_.size(); ++i)
        sql += wxT(", ") + this->data_[i].name
               + wxT(" = ") + this->sql_value_(i);
    wxString cond = wxString(condition).Trim();
    if(cond == wxEmptyString)
        cond = this->condition_;
    sql += wxT(" where ") + cond;

    return this->run_(sql);
}


void
wxPg::Updatable_row::set_field_(const wxString &field, const wxString &value,
                                bool null, int type){
    int i = this->col_idx(field);
    if(i < 0 && this->loaded())
        return;
    if(i >= 0){
        this->data_[i].value = value;
        this->data_[i].null = null;
        this->data_[i].type = type;
    }else{
        col_data new_item;
        new_item.name = field;
        new_item.value = value;
        new_item.null = null;
        new_item.type = type;
        this->data_.push_back(new_item);
        this->field_indices_[field] = this->data_.size() - 1;
    }
}


bool
wxPg::Updatable_row::delete_record(){
    if(!this->loaded()){
        this->last_err_ = _("No record loaded.");
    }
    if(!this->database()){
        this->last_err_ = _("No database connection.");
        return false;
    }
    wxString table = wxString(this->table()).Trim();
    if(table == wxEmptyString){
        this->last_err_ = _("Table not set.");
        return false;
    }
    if(this->condition_.Trim() == wxEmptyString){
        this->last_err_ = _("No condition for delete.");
        return false;
    }

    wxString sql = wxT("delete from ") + table
                   + wxT(" where ") + this->condition_;
    return this->run_(sql);
}


bool
wxPg::Updatable_row::load_(PGresult *res){
    int rows = PQntuples(res);
    if(rows > 1){
        this->last_err_ = _("More than 1 row.");
        return false;
    }
    int cols = PQnfields(res);
    if(cols <= 0){
        this->last_err_ = _("No columns.");
        return false;
    }

    this->data_.resize(cols);
    for(int i = 0; i < cols; ++i){
        this->data_[i].name = wxString::FromAscii(PQfname(res, i));
        this->data_[i].type = oid_to_type_simple(PQftype(res, i));
        this->data_[i].null = (rows == 1) ? PQgetisnull(res, 0, i) : true;
        this->field_indices_[this->data_[i].name] = i;
        if(!this->data_[i].null)
            this->data_[i].value = this->format_cached_value(
                    PQgetvalue(res, 0, i), this->data_[i].type,
                    *this->database());
    }

    return true;
}


wxString
wxPg::Updatable_row::sql_value_(int idx){
    if(this->data_[idx].null)
        return wxT("NULL");
    wxString value = this->data_[idx].value;
    if(this->data_[idx].type == TYPE_STRING)
        value = escape_string(*this->database(), value);
    return wxT("'") + value + wxT("'");
}


bool
wxPg::Updatable_row::modify_check_(){
    if(!this->database()){
        this->last_err_ = _("No database connection.");
        return false;
    }
    if(!this->cols()){
        this->last_err_ = _("No columns defined.");
        return false;
    }
    if(wxString(this->table()).Trim() == wxEmptyString){
        this->last_err_ = _("Table not set.");
        return false;
    }

    return true;
}


bool
wxPg::Updatable_row::run_(const wxString &sql){
    bool result = this->database()->run(sql);
    if(result)
        this->last_err_ = wxEmptyString;
    else this->last_err_ = this->database()->last_err();
    return result;
}
