/* grid_table.cpp - implementation of class wxPg::Grid_table - grid table.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#include<wxPg/widgets/base/grid_table.h>
#include<wxPg/utils/type.h>


wxPg::Grid_table::Grid_table()
: query_(NULL), use_midle_fetch_(false), bind_defs_(NULL),
  delete_bind_defs_(false)
{}


wxPg::Grid_table::Grid_table(wxPg::Database *db)
: query_(db), use_midle_fetch_(false), bind_defs_(NULL),
  delete_bind_defs_(false)
{}


wxPg::Grid_table::~Grid_table(){
    if(this->bind_defs_ && this->delete_bind_defs_)
        delete this->bind_defs_;
}


void
wxPg::Grid_table::bind_defs(Grid_bind_def *defs, bool delete_defs){
    if(this->bind_defs_ && delete_bind_defs_)
        delete this->bind_defs_;
    this->bind_defs_ = defs;
    this->delete_bind_defs_ = delete_defs;
}


wxPg::Grid_bind_def*
wxPg::Grid_table::bind_defs(){
    this->make_bind_defs_();
    return this->bind_defs_;
}


void
wxPg::Grid_table::bind(int col_idx, const wxString &field){
    this->make_bind_defs_();
    this->bind_defs_->bind(col_idx, field);
}


void
wxPg::Grid_table::bind(const wxString &field){
    this->make_bind_defs_();
    this->bind_defs_->bind(field);
}


void
wxPg::Grid_table::bind(const std::map<int, wxString> &col_bindings){
    this->make_bind_defs_();
    this->bind_defs_->bind(col_bindings);
}


const std::map<int, wxString>
wxPg::Grid_table::bindings()const{
    if(this->bind_defs_)
        return this->bind_defs_->bindings();
    return std::map<int, wxString>();
}


void
wxPg::Grid_table::unbind(int col_idx){
    if(this->bind_defs_)
        this->bind_defs_->unbind(col_idx);
}


void
wxPg::Grid_table::unbind(){
    if(this->bind_defs_)
        this->bind_defs_->unbind();
}


void
wxPg::Grid_table::col_label(const wxString &field, const wxString &label){
    this->make_bind_defs_();
    this->bind_defs_->col_label(field, label);
}


void
wxPg::Grid_table::rem_col_label(const wxString &field){
    if(this->bind_defs_)
        this->bind_defs_->rem_col_label(field);
}


void
wxPg::Grid_table::col_labels(const std::map<wxString, wxString> &labels){
    this->make_bind_defs_();
    this->bind_defs_->col_labels(labels);
}


const std::map<wxString, wxString>&
wxPg::Grid_table::col_labels(){
    this->make_bind_defs_();
    return this->bind_defs_->col_labels();
}


void
wxPg::Grid_table::col_labels_clear(){
    if(this->bind_defs_)
        this->bind_defs_->col_labels_clear();
}


void
wxPg::Grid_table::col_attr_define(const wxString &field, wxGridCellAttr *attr){
    if(!this->bind_defs_)
        return;
    int col = this->bind_defs_->field_binding(field);
    if(col < 0)
        return;
    this->SetColAttr(attr, col);
}


int
wxPg::Grid_table::GetNumberRows(){
    return this->query().record_count();
}


int
wxPg::Grid_table::GetNumberCols(){
    this->make_bind_defs_();
    int num_cols = this->bind_defs_->col_count();
    if(num_cols)
        return num_cols;
    return this->query().cols();
}


bool
wxPg::Grid_table::CanHaveAttributes(){
    return true;
}


bool
wxPg::Grid_table::IsEmptyCell(int row, int col){
    if(!this->query().go_to_record(row, this->fetch_buf_pos_()))
        return true;
    this->make_bind_defs_();
    std::map<int, wxString>::const_iterator
    it = this->bind_defs_->bindings().find(col);
    if(it != this->bind_defs_->bindings().end())
        return this->query().null(it->second);
    if(this->query().null(col))
        return true;
    return false;
}


wxString
wxPg::Grid_table::GetValue(int row, int col){
    return this->query().get_str(this->goto_row_col_(row, col));
}


wxString
wxPg::Grid_table::GetTypeName(int row, int col){
    col = this->goto_row_col_(row, col);
    switch(this->query().type_simple(col)){
    case wxPg::TYPE_INT:
        return wxGRID_VALUE_NUMBER;
    case wxPg::TYPE_FLOAT:
        return wxGRID_VALUE_FLOAT;
    case wxPg::TYPE_STRING:
    case wxPg::TYPE_DATE:
    case wxPg::TYPE_TIME:
    case wxPg::TYPE_TIMESTAMP:
        return wxGRID_VALUE_STRING;
    case wxPg::TYPE_BOOL:
        return wxGRID_VALUE_BOOL;
    default:
        return this->wxGridTableBase::GetTypeName(row, col);
    }
}


bool
wxPg::Grid_table::CanGetValueAs(int row, int col, const wxString &typeName){
    if(typeName == wxGRID_VALUE_NUMBER || typeName == wxGRID_VALUE_FLOAT
            || typeName == wxGRID_VALUE_STRING
            || typeName == wxGRID_VALUE_BOOL)
        return true;
    return this->wxGridTableBase::CanGetValueAs(row, col, typeName);
}


long
wxPg::Grid_table::GetValueAsLong(int row, int col){
    return this->query().get_int(this->goto_row_col_(row, col));
}


double
wxPg::Grid_table::GetValueAsDouble(int row, int col){
    return this->query().get_float(this->goto_row_col_(row, col));
}


bool
wxPg::Grid_table::GetValueAsBool(int row, int col){
    return this->query().get_bool(this->goto_row_col_(row, col));
}


wxString
wxPg::Grid_table::GetColLabelValue(int col){
    this->make_bind_defs_();
    std::map<int, wxString>::const_iterator
    it = this->bind_defs_->bindings().find(col);
    if(it != this->bind_defs_->bindings().end()){
        std::map<wxString, wxString>::const_iterator
        i = this->bind_defs_->col_labels().find(it->second);
        if(i != this->bind_defs_->col_labels().end())
            return i->second;
        return it->second;
    }

    if(this->bind_defs_->bindings().size())
        return this->wxGridTableBase::GetColLabelValue(col);
    return this->query().col_name(col);
}


//can not set value
void
wxPg::Grid_table::SetValue(int row, int col, const wxString &value){}


unsigned
wxPg::Grid_table::fetch_buf_pos_(){
    if(this->use_midle_fetch())
        return this->query().buffer_size() / 2;
    return this->query().buffer_size() - 1;
}


void
wxPg::Grid_table::make_bind_defs_(){
    if(!this->bind_defs_){
        this->bind_defs_ = new Grid_bind_def;
        this->delete_bind_defs_ = true;
    }
}


int
wxPg::Grid_table::goto_row_col_(int row, int col){
    if(!this->query().go_to_record(row, this->fetch_buf_pos_()))
        return -1;
    this->make_bind_defs_();
    std::map<int, wxString>::const_iterator
    it = this->bind_defs_->bindings().find(col);
    if(it != this->bind_defs_->bindings().end())
        return this->query().col_idx(it->second);
    if(this->bind_defs_->bindings().size())
        return -1;
    return col;
}
