/* cached_result_test.cpp - test program for wxPg::Cached_result.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#include<iostream>
#include<wx/wx.h>
#include<test_logger.h>
#include<testdb.h>
#include<wxPg/db/query.h>
#include<wxPg/db/cached_result.h>

wxPg::Query
src(&db);

wxPg::Cached_result
testee;


int
row_count, col_count;


bool
prepare_test_env(){
    if(!db_connect())
        return false;

    wxString create_table = wxT(
        "CREATE TABLE cached_result_test( \
           id integer, \
           fnum double precision, \
           create_date date, \
           create_time time without time zone, \
           create_timestamp timestamp without time zone, \
           ok boolean, \
           title text \
        );"
    );
    col_count = 7;

    if(!db.run(create_table)){
        logger.log_error("Fatal error", "Failed to create test table.");
        return false;
    }

    row_count = 0;
    bool ins_success = db.run(wxT("insert into cached_result_test values( \
                                   1, 0.5, '2000.01.01', '10:10', '2000.01.01 10:10', \
                                   true, 'aaa')"));
    row_count += ins_success ? 1 : 0;
    ins_success = ins_success && db.run(wxT("insert into cached_result_test values( \
                                             2, 0.4, '2005.05.05', '11:11', '2005.05.05 11:11', \
                                             false, 'bbb')"));
    row_count += ins_success ? 1 : 0;
    ins_success = ins_success && db.run(wxT("insert into cached_result_test values( \
                                             3, 3.3, '2007.07.07', '12:12', '2007.07.07 12:12', \
                                             false, '')"));
    row_count += ins_success ? 1 : 0;
    ins_success = ins_success && db.run(wxT("insert into cached_result_test values( \
                                             null, null, null, null, null, \
                                             null, null)"));
    row_count += ins_success ? 1 : 0;

    if(!ins_success){
        logger.log_error("Fatal error", "Failed to insert test data.");
        return false;
    }

    return true;
}


void
drop_test_table(){
    src.clear();
    db.run(wxT("drop table cached_result_test"));
}


bool
compare_2_ints(int x, int y, const char *err_title, const char *err_msg){
    if(x != y){
        wxString msg = wxString::From8BitData(err_msg) + wxT(" - ");
        msg << x << wxT(" != ") << y;
        logger.log_error("Row count", msg.char_str());
        return false;
    }

    return true;
}


#define FIELD_VALUE_COMPARISON(method, cmp_type) \
    if(src.method(field) != testee.method(field)){ \
        msg << wxT(cmp_type); \
        logger.log_error("Compare fields", msg.char_str()); \
        return false; \
    }


bool
compare_field(int field){
    wxString msg = wxT("Record ");
    msg << testee.current() << wxT(" field ") << field;
    FIELD_VALUE_COMPARISON(null, " null check failed.")
    FIELD_VALUE_COMPARISON(get_bool, " boolean comparison failed.")
    FIELD_VALUE_COMPARISON(get_date, " date comparison failed.")
    FIELD_VALUE_COMPARISON(get_date_time, " timestamp comparison failed.")
    FIELD_VALUE_COMPARISON(get_float, " float comparison failed.")
    FIELD_VALUE_COMPARISON(get_int, " int comparison failed.")
    FIELD_VALUE_COMPARISON(get_str, " string comparison failed.")
    FIELD_VALUE_COMPARISON(get_time, " time comparison failed.")

    return true;
}


bool
compare_records(){
    for(int i = 0; i < testee.cols(); ++i)
        if(!compare_field(i))
            return false;

    return true;
}


bool
compare_fields(){
    if(!compare_2_ints(src.cols(), testee.cols(), "Col count", "src|testee"))
        return false;
    else if(!compare_2_ints(testee.cols(), col_count, "Col count", "testee|real"))
        return false;
    else logger.log_success("Col count");

    for(int i = 0; i < testee.cols(); ++i){
        if(src.col_name(i) != testee.col_name(i)){
            wxString msg = wxT("Field ");
            msg << i << wxT(" - ") << src.col_name(i) << wxT(" != ") << testee.col_name(i);
            logger.log_error("Field names", msg.char_str());
            return false;
        }else if(!compare_2_ints(src.type_simple(i), testee.type_simple(i),
                                "Field types", "src|testee"))
            return false;;
    }
    logger.log_success("Field names/types");

    return true;
}


bool
compare_record_count(){
    if(!compare_2_ints(src.rows(), testee.rows(), "Row count", "src|testee"))
        return false;
    else if(!compare_2_ints(testee.rows(), row_count, "Row count", "testee|real"))
        return false;
    else logger.log_success("Row count");
    return true;
}


bool
run_tests_sub(){
    if(!compare_record_count())
        return false;
    if(!compare_fields())
        return false;
    for(int i = 0; i < testee.rows(); ++i){
        src.current(i);
        testee.current(i);
        if(!compare_records())
            return false;
    }
    logger.log_success("Records");

    return true;
}


bool
run_tests(){
    if(!src.open(wxT("select * from cached_result_test"))){
        logger.log_error("Fatal error", "Failed to select data.");
        return false;
    }else if(!src.fetch(0, wxPg::Query::FM_FORWARD_ALL)){
        logger.log_error("Fatal error", "Failed to fetch rows.");
        src.clear();
        return false;
    }
    testee.store(db, src.result());

    bool result = run_tests_sub();
    if(!result){
        src.clear();
        return false;
    }
    logger.log_check_point("Part 1");

    testee.clear();
    wxPg::Cached_result tmp;
    tmp.store(db, src.result());
    testee = tmp;
    result = run_tests_sub();
    logger.log_check_point("Part 2");
    return result;
}


int
main(int argc, char **argv){
    if(!wxInitialize()){
        std::cerr << "Initialization of wxWidgets failed!";
        return 1;
    }
    if(!prepare_test_env())
        return 2;

    int result = 0;
    if(!run_tests())
        result = 3;

    drop_test_table();
    wxUninitialize();
    logger.output_summary();
    return result;
}

