/* binding_ctrl.h - helper classes for binding wxWindow to
    wxPg::Row_binding

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __BINDING_CTRL_H
#define __BINDING_CTRL_H

#include "wxPg/conf.h"
#include "wxPg/db/data_row_provider.h"
#include "wxPg/db/updatable_row.h"

//forward declarations
#ifndef _WX_WINDOW_H_BASE_
class wxWindow;
#endif
#ifndef _WX_CALCTRL_H_
class wxCalendarCtrl;
#endif
#ifndef _WX_COMBOBOX_H_BASE_
class wxComboBox;
#endif
#ifndef _WX_COMBOCONTROL_H_BASE_
class wxComboCtrl;
#endif
#ifndef _WX_RICHTEXTCTRL_H_
class wxRichTextCtrl;
#endif
#ifndef _WX_TEXTCTRL_H_BASE_
class wxTextCtrl;
#endif
#ifndef _WX_SPINCTRL_H_
class wxSpinCtrl;
#endif
#ifndef _WX_STATTEXT_H_BASE_
class wxStaticText;
#endif
#ifndef _WX_HYPERLINK_H__
class wxHyperlinkCtrl;
#endif
#ifndef _WX_DATECTRL_H_
class wxDatePickerCtrl;
#endif
#ifndef _WX_CHECKBOX_H_BASE_
class wxCheckBox;
#endif
#ifndef _WX_GAUGE_H_BASE_
class wxGauge;
#endif
#ifndef _WX_SPINBUTT_H_BASE_
class wxSpinButton;
#endif
#ifndef _WX_BMPCBOX_H_BASE_
class wxBitmapComboBox;
#endif
#ifndef _WX_LISTBOX_H_BASE_
class wxListBox;
#endif
#ifndef _WX_CHOICE_H_BASE_
class wxChoice;
#endif
#ifndef _WX_HTMLLBOX_H_
class wxSimpleHtmlListBox;
#endif
#ifndef _WX_ODCOMBO_H_
class wxOwnerDrawnComboBox;
#endif
#ifndef _WX_RADIOBOX_H_BASE_
class wxRadioBox;
#endif
#ifndef _WX_VLBOX_H_
class wxVListBox;
#endif
#ifndef _WX_HTMLWIN_H_
class wxHtmlWindow;
#endif


namespace wxPg{

/** Base updater for window.
    Derive new window updater for any new wxWindow descendant
    you want to be supported by Row_binding.
*/
class wxPG_DECLSPEC
wxWindow_updater{
public:
    /** Constructor.
        @param win - wxWindow to be updated by this updater.
    */
    explicit
    wxWindow_updater(wxWindow *win);

    /** Destructor. */
    virtual
    ~wxWindow_updater(){}

    /** Update control.
        This method should be overrided in derived class.
        @param win - wxWindow to be updated.
        @param data - row to obtain value from.
     */
    virtual void
    operator()(Data_row_provider *data) = 0;

    /** Store value to field.
        Override this method in derived class to provide update functionality.
        @param row - updatable row, containing field to update.
    */
    virtual void
    operator()(Updatable_row *row){}

    /** Should this object be deleted by binding object.
        This method indicates, whether binding object is responsible for
        deleting this object.
        @return true to delete, false to keep.
    */
    virtual bool
    delete_me(){return true;}

    /** Get current window, updated by this updater.
        @return currenly used window.
    */
    wxWindow*
    window(){return this->win_;}
private:
    wxWindow*
    win_;
};


/** Default updater for wxWindow.
    Base class for other wxWindow updaters.
*/
class wxPG_DECLSPEC
wxWindow_def_updater : public wxWindow_updater{
public:
    /** Constructor.
        @param win - wxWindow to be updated by this updater.
        @param field - database field name.
    */
    wxWindow_def_updater(wxWindow *win, const wxString &field);

    virtual
    ~wxWindow_def_updater(){}

    /** Change field.
        @param field - new database field name.
    */
    void
    field(const wxString &field){this->field_ = field;}

    /** Get current field.
        @return current database field name.
    */
    const wxString&
    field()const{return this->field_;}
private:
    wxString field_;
};




/** Updater for wxCalendarCtrl. */
class wxPG_DECLSPEC
wxCalendarCtrl_updater : public wxWindow_def_updater{
public:
    /** Constructor.
        @param calendar - wxCalendarCtrl to be updated.
        @param field - database field name.
    */
    wxCalendarCtrl_updater(wxCalendarCtrl *calendar, const wxString &field);

    virtual
    ~wxCalendarCtrl_updater(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);

    //inherited
    virtual void
    operator()(Updatable_row *row);
};



/** Updater for wxComboBox. */
class wxPG_DECLSPEC
wxComboBox_updater : public wxWindow_def_updater{
public:
    /** Constructor.
        @param combo - wxComboBox to be updated.
        @param field - database field name.
    */
    wxComboBox_updater(wxComboBox *combo, const wxString &field);

    virtual
    ~wxComboBox_updater(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);

    //inherited
    virtual void
    operator()(Updatable_row *row);
};



/** Updater for wxComboCtrl. */
class wxPG_DECLSPEC
wxComboCtrl_updater : public wxWindow_def_updater{
public:
    /** Constructor.
        @param combo - wxComboCtrl to be updated.
        @param field - database field name.
    */
    wxComboCtrl_updater(wxComboCtrl *combo, const wxString &field);

    virtual
    ~wxComboCtrl_updater(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);

    //inherited
    virtual void
    operator()(Updatable_row *row);
};



/** Updater for wxRichTextCtrl. */
class wxPG_DECLSPEC
wxRichTextCtrl_updater : public wxWindow_def_updater{
public:
    /** Constructor.
        @param text - wxRichTextCtrl to be updated.
        @param field - database field name.
    */
    wxRichTextCtrl_updater(wxRichTextCtrl *text, const wxString &field);

    virtual
    ~wxRichTextCtrl_updater(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);

    //inherited
    virtual void
    operator()(Updatable_row *row);
};



/** Updater for wxTextCtrl. */
class wxPG_DECLSPEC
wxTextCtrl_updater : public wxWindow_def_updater{
public:
    /** Constructor.
        @param text - wxTextCtrl to be updated.
        @param field - database field name.
    */
    wxTextCtrl_updater(wxTextCtrl *text, const wxString &field);

    virtual
    ~wxTextCtrl_updater(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);

    //inherited
    virtual void
    operator()(Updatable_row *row);
};



/** Updater for wxSpinCtrl. */
class wxPG_DECLSPEC
wxSpinCtrl_updater : public wxWindow_def_updater{
public:
    /** Constructor.
        @param spin - wxSpinCtrl to be updated.
        @param field - database field name.
    */
    wxSpinCtrl_updater(wxSpinCtrl *spin, const wxString &field);

    virtual
    ~wxSpinCtrl_updater(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);

    //inherited
    virtual void
    operator()(Updatable_row *row);
};




/** Updater for wxStaticText. */
class wxPG_DECLSPEC
wxStaticText_updater : public wxWindow_def_updater{
public:
    /** Constructor.
        @param text - wxStaticText to be updated.
        @param field - database field name.
    */
    wxStaticText_updater(wxStaticText *text, const wxString &field);

    virtual
    ~wxStaticText_updater(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);
};




/** Updater for wxHyperlinkCtrl.
    This updater updates only URL.
*/
class wxPG_DECLSPEC
wxHyperlinkCtrl_updater : public wxWindow_def_updater{
public:
    /** Constructor.
        @param hyperlink - wxHyperlinkCtrl to be updated.
        @param field - database field name.
    */
    wxHyperlinkCtrl_updater(wxHyperlinkCtrl *hyperlink, const wxString &field);

    virtual
    ~wxHyperlinkCtrl_updater(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);
};




/** Updater for wxHyperlinkCtrl with label update.
    This updater updates both URL and label.
*/
class wxPG_DECLSPEC
wxHyperlinkCtrl_updater_wlabel : public wxHyperlinkCtrl_updater{
public:
    /** Constructor
        @param hyperlink - wxHyperlinkCtrl to be updated.
        @param url_field - database field name for URL.
        @param label_field - database field name for label.
    */
    wxHyperlinkCtrl_updater_wlabel(wxHyperlinkCtrl *hyperlink,
                                   const wxString &url_field,
                                   const wxString &label_field);

    virtual
    ~wxHyperlinkCtrl_updater_wlabel(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);

    /** Set URL field.
        @param field - database field name for URL.
    */
    void
    url_field(const wxString &field){this->field(field);}

    /** Get URL field.
        @return database field name for URL.
    */
    const wxString&
    url_field()const{return this->field();}

    /** Set label field.
        @param field - database field name for label.
    */
    void
    label_field(const wxString &field){this->label_field_ = field;}

    /** Get label field.
        @return database field name for label.
    */
    const wxString&
    label_field()const{return this->label_field_;};
private:
    wxString
    label_field_;
};




/** Updater for wxDatePickerCtrl. */
class wxPG_DECLSPEC
wxDatePickerCtrl_updater : public wxWindow_def_updater{
public:
    /** Constructor.
        @param date_ctrl - wxDatePickerCtrl to be updated.
        @param field - database field name.
    */
    wxDatePickerCtrl_updater(wxDatePickerCtrl *date_ctrl,
                             const wxString &field);

    virtual
    ~wxDatePickerCtrl_updater(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);

    //inherited
    virtual void
    operator()(Updatable_row *row);
};




/** Updater for wxCheckBox. */
class wxPG_DECLSPEC
wxCheckBox_updater : public wxWindow_def_updater{
public:
    /** Constructor.
        @param check_box - wxCheckBox to be updated.
        @param field - database field name.
    */
    wxCheckBox_updater(wxCheckBox *check_box, const wxString &field);

    virtual
    ~wxCheckBox_updater(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);

    //inherited
    virtual void
    operator()(Updatable_row *row);
};




/** Updater for wxGauge. */
class wxPG_DECLSPEC
wxGauge_updater : public wxWindow_def_updater{
public:
    /** Constructor.
        @param gauge - wxGauge to be updated.
        @param field - database field name.
    */
    wxGauge_updater(wxGauge *gauge, const wxString &field);

    virtual
    ~wxGauge_updater(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);
};





/** Updater for wxSpinButton. */
class wxPG_DECLSPEC
wxSpinButton_updater : public wxWindow_def_updater{
public:
    /** Constructor.
        @param spin - wxSpinButton to be updated.
        @param field - database field name.
    */
    wxSpinButton_updater(wxSpinButton *spin, const wxString &field);

    virtual
    ~wxSpinButton_updater(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);

    //inherited
    virtual void
    operator()(Updatable_row *row);
};




/** Updater for various wxItemContainer decendants. */
class wxPG_DECLSPEC
wxItemContainer_updater : public wxWindow_def_updater{
public:
    /** Constructor.
        @param combo - wxBitmapComboBox to be updated.
        @param field - database field name.
    */
    wxItemContainer_updater(wxBitmapComboBox *combo, const wxString &field);

    /** Constructor.
        @param list - wxChoice to be updated.
        @param field - database field name.
    */
    wxItemContainer_updater(wxChoice *list, const wxString &field);

    /** Constructor.
        @param combo - wxComboBox to be updated.
        @param field - database field name.
    */
    wxItemContainer_updater(wxComboBox *combo, const wxString &field);

    /** Constructor.
        @param list - wxSimpleHtmlListBox to be updated.
        @param field - database field name.
    */
    wxItemContainer_updater(wxSimpleHtmlListBox *list, const wxString &field);

    /** Constructor.
        @param list - wxListBox to be updated.
        @param field - database field name.
    */
    wxItemContainer_updater(wxListBox *list, const wxString &field);

    /** Constructor.
        @param list - wxOwnerDrawnComboBox to be updated.
        @param field - database field name.
    */
    wxItemContainer_updater(wxOwnerDrawnComboBox *list, const wxString &field);

    /** Constructor.
        @param radio_box - wxRadioBox to be updated.
        @param field - database field name.
    */
    wxItemContainer_updater(wxRadioBox *radio_box, const wxString &field);

    virtual
    ~wxItemContainer_updater(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);

    //inherited
    virtual void
    operator()(Updatable_row *row);
};




/** Updater for wxRadioBox.
    This updater updates by index, rather than by string value.
*/
class wxPG_DECLSPEC
wxRadioBox_updater : public wxWindow_def_updater{
public:
    /** Constructor.
        Čparam box - wxRadioBox to be updated.
        @param field - database field name.
    */
    wxRadioBox_updater(wxRadioBox *box, const wxString &field);

    virtual
    ~wxRadioBox_updater(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);

    //inherited
    virtual void
    operator()(Updatable_row *row);
};




/** Updater for wxVListBox. */
class wxPG_DECLSPEC
wxVListBox_updater : public wxWindow_def_updater{
public:
    /** Constructor.
        @param box - wxVListBox to be updated.
        @param field - database field name.
    */
    wxVListBox_updater(wxVListBox *box, const wxString &field);

    virtual
    ~wxVListBox_updater(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);

    //inherited
    virtual void
    operator()(Updatable_row *row);
};




/** Updater for wxHtmlWindow. */
class wxPG_DECLSPEC
wxHtmlWindow_updater : public wxWindow_def_updater{
public:
    /** Constructor.
        @param html - wxHtmlWindow to be updated.
        @param field - database field name.
    */
    wxHtmlWindow_updater(wxHtmlWindow *html, const wxString &field);

    virtual
    ~wxHtmlWindow_updater(){}

    //inherited
    virtual void
    operator()(Data_row_provider *data);
};


} //namespace wxPg

#endif
