/* binding_list.h - helper classes for binding wxItemContainer to
    wxPg::List_binding

    Copyright (C) 2008, 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __BINDING_LIST_H
#define __BINDING_LIST_H

#include "wxPg/conf.h"
#include<wx/arrstr.h>
#include "wxPg/db/result_data_provider.h"

//forward declarations
#ifndef _WX_WINDOW_H_BASE_
class wxWindow;
#endif
#ifndef _WX_CTRLSUB_H_BASE_
class wxItemContainer;
#endif
#ifndef _WX_BMPCBOX_H_BASE_
class wxBitmapComboBox;
#endif
#ifndef _WX_LISTBOX_H_BASE_
class wxListBox;
#endif
#ifndef _WX_CHOICE_H_BASE_
class wxChoice;
#endif
#ifndef _WX_COMBOBOX_H_BASE_
class wxComboBox;
#endif
#ifndef _WX_HTMLLBOX_H_
class wxSimpleHtmlListBox;
#endif
#ifndef _WX_ODCOMBO_H_
class wxOwnerDrawnComboBox;
#endif
#ifndef _WX_RADIOBOX_H_BASE_
class wxRadioBox;
#endif
#ifndef _WX_VLBOX_H_
class wxVListBox;
#endif


namespace wxPg{

/** Base updater for window.
    Derive new window updater for any new wxWindow descendant
    you want to be supported by Row_binding.
*/
class
wxWindow_list_updater{
public:
    /** Constructor.
        @param win - wxWindow to be updated.
    */
    explicit
    wxWindow_list_updater(wxWindow *win);

    /** Destructor. */
    virtual
    ~wxWindow_list_updater(){}

    /** Get window, updated by this object.
        @return window, updated by this object.
    */
    wxWindow*
    window(){return this->win_;}

    /** Use as function object.
        If returns true, this object is called as function (operator()),
        to do all-at-once update.\n
        If returns false, begin_update(), update_row() and end_update()
        are used.
        @return true to be used as function object, false otherwise.
    */
    virtual bool
    use_fo()const{return false;}

    /** Start updating window.
        Called when starting update and only if use_fo() return false.
        @param data - data provider to obtain data from,
                      do restore it to the state before the call!
    */
    virtual void
    begin_update(Result_data_provider *data) = 0;

    /** Update window by current row.
        Called for each row in data, after a call to begin_update().
        Is used only if use_fo() return false.
        @param data - data provider to obtain data from,
                      do restore it to the state before the call!
    */
    virtual void
    update_row(Result_data_provider *data) = 0;

    /** End updating window.
        Called after calls to update_row().
        Is used only if use_fo() return false.
    */
    virtual void
    end_update() = 0;

    /** Update window all-at-once.
        This is called only if use_fo returns true and is called
        only once.
        @param data - data provider to obtain data from,
                      do restore it to the state before the call!
    */
    virtual void
    operator()(Result_data_provider *data) = 0;

    /** Should this object be deleted by binding object.
        This method indicates, whether binding object is responsible for
        deleting this object.
        @return true to delete, false to keep.
    */
    virtual bool
    delete_me(){return true;}
private:
    wxWindow*
    win_;
};


/** List updater for wxItemContainer. */
class wxPG_DECLSPEC
wxItemContainer_list_updater : public wxWindow_list_updater{
public:
    /** Constructor.
        @param combo - wxBitmapComboBox to be updated.
        @param field - database field name.
    */
    wxItemContainer_list_updater(wxBitmapComboBox *combo,
                                 const wxString &field);

    /** Constructor.
        @param list - wxChoice to be updated.
        @param field - database field name.
    */
    wxItemContainer_list_updater(wxChoice *list, const wxString &field);

    /** Constructor.
        @param combo - wxComboBox to be updated.
        @param field - database field name.
    */
    wxItemContainer_list_updater(wxComboBox *combo, const wxString &field);

    /** Constructor.
        @param list - wxSimpleHtmlListBox to be updated.
        @param field - database field name.
    */
    wxItemContainer_list_updater(wxSimpleHtmlListBox *list,
                                 const wxString &field);

    /** Constructor.
        @param list - wxListBox to be updated.
        @param field - database field name.
    */
    wxItemContainer_list_updater(wxListBox *list, const wxString &field);

    /** Constructor.
        @param list - wxOwnerDrawnComboBox to be updated.
        @param field - database field name.
    */
    wxItemContainer_list_updater(wxOwnerDrawnComboBox *list,
                                 const wxString &field);

    /** Constructor.
        @param radio_box - wxRadioBox to be updated.
        @param field - database field name.
    */
    wxItemContainer_list_updater(wxRadioBox *radio_box, const wxString &field);

    virtual
    ~wxItemContainer_list_updater(){}

    /** Change field.
        @param field - new database field name.
    */
    void
    field(const wxString &field){this->field_ = field;}

    /** Get current field.
        @return current database field name.
    */
    const wxString
    field()const{return this->field_;}

    //inherited
    virtual void
    begin_update(Result_data_provider *data);

    //inherited
    virtual void
    update_row(Result_data_provider *data);

    //inherited
    virtual void
    end_update();

    //inherited
    virtual void
    operator()(Result_data_provider *data);
private:
    wxString
    field_;

    wxArrayString
    tmp_values_;
};


}

#endif
