/* grid_bind_def.h - class wxPg::Grid_bind_def for containing grid binding
    definitions.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __GRID_BIND_DEF_H
#define __GRID_BIND_DEF_H


#include "wxPg/conf.h"
#include<map>
#include<wx/string.h>

namespace wxPg{

/** Grid binding definitions. */
class wxPG_DECLSPEC
Grid_bind_def{
public:
    /** Constructor. */
    Grid_bind_def(){}

    /** Constructor.
        @param col_bindings - column bindings to be set (see bind()).
    */
    explicit
    Grid_bind_def(const std::map<int, wxString> &col_bindings);

    /** Constructor.
        @param col_bindings - column bindings to be set.
        @param labels - column labels (see col_labels()).
    */
    Grid_bind_def(const std::map<int, wxString> &col_bindings,
                  const std::map<wxString, wxString> &labels);

    /** Destructor. */
    virtual
    ~Grid_bind_def(){}

    /** Bind grid column to database field.
        @param col_idx - grid columns index.
        @param field - database field. Grid column will contain values from
        this field.
    */
    void
    bind(int col_idx, const wxString &field){
        this->col_bindings_[col_idx] = field;
    }

    /** Bind next grid column to database field.
        The newly binded column has index, that is bigger by 1
        than currenly binded column with highest index.
        It's not recomended to mix this method with other binding methods.
    */
    void
    bind(const wxString &field);

    /** Bind grid columns to database fields.
        @param col_bindings - keys are column indices, values are database
        fields binded to them.
    */
    void
    bind(const std::map<int, wxString> &col_bindings){
        this->col_bindings_ = col_bindings;
    }

    /** Return current bindings for grid.
        Keys in map are column indices, values are database
        fields binded to them.
        @return current bindings.
    */
    const std::map<int, wxString>&
    bindings()const{return this->col_bindings_;}

    /** Remove column from current bindings.
        @param col_idx - grid column index.
    */
    void
    unbind(int col_idx){this->col_bindings_.erase(col_idx);}

    /** Remove all binded columns from list. */
    void
    unbind(){this->col_bindings_.clear();}

    /** Get index of column, to which the field is binded.
        If field is not binded, -1 is returned.
        @param field - field name.
        @return column index.
    */
    int
    field_binding(const wxString &field)const;

    /** Set label for grid column, binded to database field.
        For columns, that don't have specified labels, field name is
        used instead.
        If no field is binded to column, label of that column is not changed.
        @param field - database field. Grid column, that is binded to this
        field will have specified label.
        @param label - label for column.
    */
    void
    col_label(const wxString &field, const wxString &label){
        this->col_labels_[field] = label;
    }

    /** Remove assigned label from column, displaying values of database field.
        @param field - database field.
    */
    void
    rem_col_label(const wxString &field){this->col_labels_.erase(field);}

    /** Set labels for grid columns.
        @param labels - keys are fied names, values are labels for
        grid columns.
    */
    void
    col_labels(const std::map<wxString, wxString> &labels){
        this->col_labels_ = labels;
    }

    /** Returns currently specified labels for fields.
        Keys in map are fied names, values are labels.
        @return current column labels.
    */
    const std::map<wxString, wxString>&
    col_labels(){return this->col_labels_;}

    /** Clears all specified labels for grid columns. */
    void
    col_labels_clear(){this->col_labels_.clear();}

    /** Clear all bindings. */
    void
    clear(){this->unbind(); this->col_labels_clear();}

    /** Get number of columns.
        @return number of columns.
    */
    unsigned
    col_count()const;

    /** Get column label by column index.
        If label is set for column, that label is returned.
        If label is not set, than field name is returned.
        If neigther label, nor field is set, empty string is returned.
        @param col_idx - column index.
        @return defined label for column.
    */
    const wxString
    col_label(unsigned col_idx)const;
private:
    std::map<int, wxString>
    col_bindings_; //column bindings

    std::map<wxString, wxString>
    col_labels_;   //column labels
};

}

#endif
