/* updatable_row_provider.h - class wxPg::Updatable_row for
    changing a single row of a database table.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __UPDATABLE_ROW_H
#define __UPDATABLE_ROW_H

#include "wxPg/conf.h"
#include "wxPg/db/database.h"
#include "wxPg/db/cached_row_provider.h"
#include<vector>
#include<map>

namespace wxPg{

/** Class for inserting, updating or deleting a single row in a table. */
class wxPG_DECLSPEC
Updatable_row : public Cached_row_provider{
public:
    /** Constructor.
        @param db - database connection.
    */
    Updatable_row(Database *db);

    /** Destructor. */
    virtual
    ~Updatable_row(){}

    /** Set database connection.
        @param db - database connection.
    */
    void
    database(Database *db){this->db_ = db;}

    /** Get database connection.
        @return database connection, used by this object.
    */
    Database*
    database(){return this->db_;}

    /** Are fields loaded.
        @return true, if fields are loaded, false otherwise.
    */
    bool
    loaded()const{return this->loaded_;};

    //inherited
    virtual int
    cols();

    //do not hide parent methods
    using Cached_row_provider::type_simple;
    using Cached_row_provider::get_str;
    using Cached_row_provider::null;

    //inherited
    virtual const wxString
    col_name(int idx);

    //inherited
    virtual int
    col_idx(const wxString &field);

    //inherited
    virtual int
    type_simple(int idx);

    //inherited
    virtual bool
    null(int idx);

    //inherited
    virtual const wxString
    get_str(int idx);

    /** Set field value.
        If fields are loaded, only an existing field can be set.
        Otherwise a field is added, if it does not exist.\n
        Also sets field type to TYPE_INT.
        @param field - fiend name.
        @param value - value.
    */
    void
    set_value(const wxString &field, int value);

    /** Set field value.
        If fields are loaded, only an existing field can be set.
        Otherwise a field is added, if it does not exist.\n
        Also sets field type to TYPE_FLOAT.
        @param field - fiend name.
        @param value - value.
    */
    void
    set_value(const wxString &field, double value);

    /** Set field value.
        If fields are loaded, only an existing field can be set.
        Otherwise a field is added, if it does not exist.\n
        Also sets field type to TYPE_STRING.
        @param field - fiend name.
        @param value - value.
    */
    void
    set_value(const wxString &field, const wxString &value);

    /** Set field value.
        If fields are loaded, only an existing field can be set.
        Otherwise a field is added, if it does not exist.\n
        Note, that this sets field type to TYPE_TIMESTAMP.
        Use set_time or set_date for only date or time.
        @param field - fiend name.
        @param value - value.
    */
    void
    set_value(const wxString &field, const wxDateTime &value);

    /** Set field value as date.
        If fields are loaded, only an existing field can be set.
        Otherwise a field is added, if it does not exist.\n
        Also sets field type to TYPE_DATE.
        @param field - fiend name.
        @param value - value.
    */
    void
    set_date(const wxString &field, const wxDateTime &value);

    /** Set field value as time.
        If fields are loaded, only an existing field can be set.
        Otherwise a field is added, if it does not exist.\n
        Also sets field type to TYPE_TIME.
        @param field - fiend name.
        @param value - value.
    */
    void
    set_time(const wxString &field, const wxDateTime &value);

    /** Set field value.
        If fields are loaded, only an existing field can be set.
        Otherwise a field is added, if it does not exist.\n
        Also sets field type to TYPE_BOOL.
        @param field - fiend name.
        @param value - value.
    */
    void
    set_bool(const wxString &field, bool value);

    /** Set field to NULL.
        If fields are loaded, only an existing field can be set.
        Otherwise a field is added, if it does not exist.\n
        If new field is added, its type is set to TYPE_UNKNOWN.
        @param field - fiend name.
    */
    void
    set_null(const wxString &field);

    /** Set type for field.
        If fields are loaded, only an existing field can be set.
        Otherwise a field is added, if it does not exist.\n
        If new field is added, ist value is set to NULL.\n
        Field value is set to NULL, if value is not acceptable for new type.\n
        For fields with type TYPE_UNKNOWN, a string value is passed varbatim
        to database.
        @param field - field name.
        @param type - data type for field.
    */
    void
    set_type_simple(const wxString &field, int type);

    /** Clear any defined fields.
        If fields are loaded, they are all set to NULL.
        Otherwise fields are removed from list.
    */
    virtual void
    clear();

    /** Set table to work with.
        @param tab - table name.
    */
    void
    table(const wxString &tab){this->table_ = tab;}

    /** Get table.
        @return table name.
    */
    const wxString&
    table()const{return this->table_;}

    /** Load data from database.
        If condition is empty or there are no rows for it,
        only column definitions are loaded.
        Otherwise a first row for a condition is loaded.\n
        If an error occurs, call last_err() to get message for it.
        @param condition - a condition for table. Should return only one row.
        @return true on success, false on error.
    */
    virtual bool
    load(const wxString &condition = wxEmptyString);

    /** Remove any loaded information and data. */
    virtual void
    unload();

    /** Get last error.
        @return error message for last error.
    */
    const wxString&
    last_err(){return this->last_err_;}

    /** Insert row to database.
        Currently cached row is inserted to database.
        If an error occurs, call last_err() to get message for it.
        @return true on success, false on error.
    */
    virtual bool
    insert();

    /** Update row in database.
        Currenly cached row is updated in database.
        Condition for update is eighter passed as argument,
        or is used the same, that was used when loading row.
        It is users responsibility to ensure, that only one row in table
        matches condition.\n
        If an error occurs, call last_err() to get message for it.
        @param condition - a condition to be used instead of load condition.
        @return true on success, false on error.
    */
    virtual bool
    update(const wxString &condition = wxEmptyString);

    /** Delete row from database.
        Deletes currenly loaded row from database.
        If an error occurs, call last_err() to get message for it.
        @return true on success, false on error.
    */
    virtual bool
    delete_record();
protected:
    wxString
    last_err_;
private:
    Database*
    db_;

    struct
    col_data{
        wxString name, value;
        bool null;
        int type;
    };

    std::vector<col_data>
    data_;

    std::map<wxString, int>
    field_indices_;

    bool
    loaded_;

    wxString
    table_, condition_;

    //set field value
    void
    set_field_(const wxString &field, const wxString &value,
               bool null, int type);

    //load data from PGresult
    bool
    load_(PGresult *res);

    //convert cached value to value, usable in SQL statement
    wxString
    sql_value_(int idx);

    //check if possible to modify record in database
    inline bool
    modify_check_();

    //run SQL statement
    inline bool
    run_(const wxString &sql);

    Updatable_row(const Updatable_row&){}
    Updatable_row &operator=(const Updatable_row&){return *this;}
};

}

#endif
