/* grid_table.h - class wxPg::Grid_table - grid table.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __GRID_TABLE_H
#define __GRID_TABLE_H

#include "wxPg/conf.h"
#include "wx/grid.h"
#include "wxPg/db/query_browser.h"
#include "wxPg/binding/grid_bind_def.h"


namespace wxPg{

/** A table for wxGrid.
    Pass it as argument to wxGrid::SetTable().
    Internally uses wxPg::Query_browser.
*/
class wxPG_DECLSPEC
Grid_table : public wxGridTableBase{
public:
    /** Create new object.
        Call database() later to set database for this object.
    */
    Grid_table();

    /** Create new object.
        @param db - database connection.
    */
    Grid_table(Database *db);

    /** Destructor. */
    virtual
    ~Grid_table();

    /** Set database connection for this object.
        @param db - database connection.
    */
    void
    database(Database *db){this->query_.database(db);}

    /** Get database, used by this object.
        @return database connection.
    */
    Database*
    database(){return this->query_.database();}

    /** Get internally used wxPg::Query_browser.
        @return query browser, used internally.
    */
    Query_browser&
    query(){return this->query_;}

    /** Fetch buffer so, that neccessairy record is in the midle.
        If passed false, that neccessairy record is at the end.
        @param use - true to use midle fetch strategy.
    */
    void
    use_midle_fetch(bool use){
        this->use_midle_fetch_ = use;
    }

    /** Return current fetch strategy.
        See setter use_midle_fetch() for more details.
    */
    bool
    use_midle_fetch()const{return this->use_midle_fetch_;}

    /** Set binding definitions for this object.
        If no definitions are set, all columns are displayed.
        @param defs - binding definitions to be used.
        @param delete_defs - this object is responsible for deleting
                             definitions.
    */
    void
    bind_defs(Grid_bind_def *defs, bool delete_defs = true);

    /** Return current binding definitions.
        @return binding definitions.
    */
    Grid_bind_def*
    bind_defs();

    /** Bind grid column to database field.
        @param col_idx - grid columns index.
        @param field - database field. Grid column will contain values from
        this field.
    */
    void
    bind(int col_idx, const wxString &field);

    /** Bind next grid column to database field.
        The newly binded column has index, that is bigger by 1
        than currenly binded column with highest index.
        It's not recomended to mix this method with other binding methods.
    */
    void
    bind(const wxString &field);

    /** Bind grid columns to database fields.
        @param col_bindings - keys are column indices, values are database
        fields binded to them.
    */
    void
    bind(const std::map<int, wxString> &col_bindings);

    /** Return current bindings for grid.
        Keys in map are column indices, values are database
        fields binded to them.
        @return current bindings.
    */
    const std::map<int, wxString>
    bindings()const;

    /** Remove column from current bindings.
        @param col_idx - grid column index.
    */
    void
    unbind(int col_idx);

    /** Remove all binded columns from list. */
    void
    unbind();

    /** Set label for grid column, binded to database field.
        For columns, that don't have specified labels, field name is
        used instead.
        If no field is binded to column, label of that column is not changed.
        @param field - database field. Grid column, that is binded to this
        field will have specified label.
        @param label - label for column.
    */
    void
    col_label(const wxString &field, const wxString &label);

    /** Remove assigned label from column, displaying values of database field.
        @param field - database field.
    */
    void
    rem_col_label(const wxString &field);

    /** Set labels for grid columns.
        @param labels - keys are fied names, values are labels for
        grid columns.
    */
    void
    col_labels(const std::map<wxString, wxString> &labels);

    /** Returns currently specified labels for fields.
        Keys in map are fied names, values are labels.
        @return current column labels.
    */
    const std::map<wxString, wxString>&
    col_labels();

    /** Clears all specified labels for grid columns. */
    void
    col_labels_clear();

    /** Clear all bindings. */
    void
    clear(){this->unbind(); this->col_labels_clear();}

    /** Define attribute for field.
        This attribute will be set for column, which will values of that field.
        @param field - field name.
        @param attr - attribute. Takes ownership.
    */
    void
    col_attr_define(const wxString &field, wxGridCellAttr *attr);

    //inherited
    virtual int
    GetNumberRows();

    //inherited
    virtual int
    GetNumberCols();

    //inherited
    virtual bool
    CanHaveAttributes();

    //inherited
    virtual bool
    IsEmptyCell(int row, int col);

    //inherited
    virtual wxString
    GetValue(int row, int col);

    //inherited
    virtual void
    SetValue(int row, int col, const wxString &value);

    //inherited
    virtual wxString
    GetTypeName(int row, int col);

    //inherited
    virtual bool
    CanGetValueAs(int row, int col, const wxString &typeName);

    //inherited
    virtual long
    GetValueAsLong(int row, int col);

    //inherited
    virtual double
    GetValueAsDouble(int row, int col);

    //inherited
    virtual bool
    GetValueAsBool(int row, int col);

    //inherited
    virtual wxString
    GetColLabelValue(int col);
private:
    //query for database data access
    Query_browser
    query_;

    //binding definitions
    Grid_bind_def*
    bind_defs_;


    bool
    use_midle_fetch_,  //fetch strategy. See use_midle_fetch()
    delete_bind_defs_; //delete binding definitions

    //Buffer position for Query_browser::go_to_record().
    inline unsigned
    fetch_buf_pos_();

    inline void
    make_bind_defs_();

    //move to row and return field
    inline int
    goto_row_col_(int row, int col);

    Grid_table(const Grid_table&) : query_(NULL){}
    Grid_table &operator=(const Grid_table&){return *this;}
};

}

#endif
