/* grid.h - class wxPg::Grid - grid widget.

    Copyright (C) 2009 Aurimas Černius

    This file is part of wxPostgres.

    wxPostgres is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    wxPostgres is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with wxPostgres.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef __GRID_H
#define __GRID_H

#include "wxPg/conf.h"
#include<wx/grid.h>
#include "wxPg/widgets/base/grid_table.h"

namespace wxPg{

/** Grid for displaying a result of SQL query. */
class Grid : public wxGrid{
public:
    /** Create new grid.
        Call Create() later and database(), to set database connection.
    */
    Grid();

    /** Create new grid.
        For argument description see wxWidgets help.
    */
    Grid(wxWindow *parent, wxWindowID id,
         const wxPoint &pos = wxDefaultPosition,
         const wxSize &size = wxDefaultSize, long style = wxWANTS_CHARS,
         const wxString &name = wxPanelNameStr);

    /** Destructor.*/
    virtual
    ~Grid();

    /** Set database connection.
        @param db - database connection.
    */
    void
    database(Database *db){this->table_.database(db);}

    /** Get database connection used by this object.
        @return database connection.
    */
    Database*
    database(){return this->table_.database();}

    /** Open SQL statement.
        Open SQL statement and refreshed grid to display data.
        @return true on success, false on error.
    */
    bool
    open(const wxString &sql);

    /** Get last error.
        Call this if open() fails.
        @return last SQL error message.
    */
    const wxString
    last_err(){return this->table_.query().last_err();}

    /** Set binding definitions for this object.
        If no definitions are set, all columns are displayed.
        @param defs - binding definitions to be used.
        @param delete_defs - this object is responsible for deleting
                             definitions.
    */
    void
    bind_defs(Grid_bind_def *defs, bool delete_defs = true){
        this->table_.bind_defs(defs, delete_defs);
    }

    /** Return current binding definitions.
        @return binding definitions.
    */
    Grid_bind_def*
    bind_defs(){return this->table_.bind_defs();}

    /** Bind grid column to database field.
        @param col_idx - grid columns index.
        @param field - database field. Grid column will contain values from
        this field.
    */
    void
    bind(int col_idx, const wxString &field){
        this->table_.bind(col_idx, field);
    }

    /** Bind next grid column to database field.
        The newly binded column has index, that is bigger by 1
        than currenly binded column with highest index.
        It's not recomended to mix this method with other binding methods.
    */
    void
    bind(const wxString &field){this->table_.bind(field);}

    /** Bind grid columns to database fields.
        @param col_bindings - keys are column indices, values are database
        fields binded to them.
    */
    void
    bind(const std::map<int, wxString> &col_bindings){
        this->table_.bind(col_bindings);
    }

    /** Return current bindings for grid.
        Keys in map are column indices, values are database
        fields binded to them.
        @return current bindings.
    */
    const std::map<int, wxString>
    bindings()const{return this->table_.bindings();}

    /** Remove column from current bindings.
        @param col_idx - grid column index.
    */
    void
    unbind(int col_idx){this->table_.unbind(col_idx);}

    /** Remove all binded columns from list. */
    void
    unbind(){this->table_.unbind();}

    /** Set label for grid column, binded to database field.
        For columns, that don't have specified labels, field name is
        used instead.
        If no field is binded to column, label of that column is not changed.
        @param field - database field. Grid column, that is binded to this
        field will have specified label.
        @param label - label for column.
    */
    void
    col_label(const wxString &field, const wxString &label){
        this->table_.col_label(field, label);
    }

    /** Remove assigned label from column, displaying values of database field.
        @param field - database field.
    */
    void
    rem_col_label(const wxString &field){this->table_.rem_col_label(field);}

    /** Set labels for grid columns.
        @param labels - keys are fied names, values are labels for
        grid columns.
    */
    void
    col_labels(const std::map<wxString, wxString> &labels){
        this->table_.col_labels(labels);
    }

    /** Returns currently specified labels for fields.
        Keys in map are fied names, values are labels.
        @return current column labels.
    */
    const std::map<wxString, wxString>&
    col_labels(){return this->table_.col_labels();}

    /** Clears all specified labels for grid columns. */
    void
    col_labels_clear(){this->table_.col_labels_clear();}

    /** Clear all bindings. */
    void
    clear(){this->unbind(); this->col_labels_clear();}

    /** Define attribute for field.
        This attribute will be set for column, which will values of that field.
        @param field - field name.
        @param attr - attribute. Takes ownership.
    */
    void
    col_attr_define(const wxString &field, wxGridCellAttr *attr){
        this->table_.col_attr_define(field, attr);
    }
private:
    Grid_table
    table_;

    Grid(const Grid&){}
    const Grid &operator=(const Grid&){return *this;}
};

}

#endif
